// file      : structure-libxml2.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006 Code Synthesis Tools CC
// license   : GNU GPL v2; see accompanying LICENSE file

#include <iostream>
#include <fstream>

#include <libxml/xmlreader.h>
#include <libxml/xmlschemas.h>

#include "time.hxx"

using namespace std;

void
process_node(xmlTextReaderPtr reader)
{
  /*
  const xmlChar *name, *value;

  name = xmlTextReaderConstName(reader);

  if (name == NULL)
    name = BAD_CAST "--";

  value = xmlTextReaderConstValue(reader);

  printf("%d %d %s %d %d",
         xmlTextReaderDepth(reader),
         xmlTextReaderNodeType(reader),
         name,
         xmlTextReaderIsEmptyElement(reader),
         xmlTextReaderHasValue(reader));

  if (value == NULL)
    printf("\n");
  else
  {
    if (xmlStrlen(value) > 40)
      printf(" %.40s...\n", value);
    else
      printf(" %s\n", value);
  }
  */
}

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " strcuture.xml" << endl;
    return 1;
  }

  try
  {
    ifstream ifs;
    ifs.exceptions (ios_base::failbit);
    ifs.open (argv[1], ios::in | ios::ate);

    size_t size (ifs.tellg ());
    ifs.seekg (0, ios::beg);

    char* buf = new char[size];
    ifs.read (buf, size);

    cerr << "document size: " << size << " bytes" << endl;

    // Initialize the library and check potential ABI mismatches
    // between the version it was compiled for and the actual shared
    // library used.
    //
    LIBXML_TEST_VERSION

    xmlSchemaParserCtxt* pctx (xmlSchemaNewParserCtxt ("structure.xsd"));
    xmlSchema* schema (xmlSchemaParse (pctx));
    xmlSchemaFreeParserCtxt (pctx);


    //
    //
    xmlTextReader* reader (xmlReaderForMemory (buf, size, argv[1], 0, 0));
    xmlTextReaderSetSchema (reader, schema);

    OS::Time start;

    for (unsigned long i (0); i < 10000; ++i)
    {
      int r (xmlTextReaderRead (reader));

      while (r == 1)
      {
        process_node (reader);
        r = xmlTextReaderRead (reader);
      }

      xmlReaderNewMemory (reader, buf, size, argv[1], 0, 0);
    }

    OS::Time end;

    cerr << "time: " << end - start << endl;

    xmlFreeTextReader (reader);
    xmlSchemaFree (schema);
    xmlCleanupParser ();
    delete[] buf;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
