// file      : structure-xdk.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006 Code Synthesis Tools CC
// license   : GNU GPL v2; see accompanying LICENSE file

#include <iostream>
#include <fstream>

extern "C"
{
#include <xml.h>
}

#include "time.hxx"

using namespace std;

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " strcuture.xml" << endl;
    return 1;
  }

  try
  {
    ifstream ifs;
    ifs.exceptions (ios_base::failbit);
    ifs.open (argv[1], ios::in | ios::ate);

    size_t size (ifs.tellg ());
    ifs.seekg (0, ios::beg);

    char* buf = new char[size];
    ifs.read (buf, size);

    cerr << "document size: " << size << " bytes" << endl;

    //
    //
    xmlerr xerr;
    xmlctx* xml_ctx (XmlCreate (&xerr, (oratext*)("structure-xdk"), 0));
    xsdctx* xsd_ctx (XmlSchemaCreate (xml_ctx, &xerr, 0));

    XmlSchemaLoad (xsd_ctx, (oratext*)("structure.xsd"), ub4 (0));


    OS::Time start;

    for (unsigned long i (0); i < 10000; ++i)
    {
      xmldocnode* doc (XmlLoadDom (xml_ctx,
                                   &xerr,
                                   "buffer", buf,
                                   "buffer_length", size,
                                   0));

      xmlnode* root (XmlDomGetDocElem (xml_ctx, doc));

      XmlSchemaValidate (xsd_ctx, xml_ctx, root);

      XmlFreeDocument (xml_ctx, doc);
    }

    OS::Time end;

    cerr << "time: " << end - start << endl;

    XmlSchemaDestroy (xsd_ctx);
    XmlDestroy (xml_ctx);
    delete[] buf;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
